@@ -14,20 +14,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import gflags
-import logging
+from oslo.config import cfg
 import webob.dec
 
 from gevent import pywsgi
 from routes import Mapper
 from routes.util import URLGenerator
 
-LOG = logging.getLogger('ryu.app.wsgi')
-
-FLAGS = gflags.FLAGS
-gflags.DEFINE_string('wsapi_host', '', 'webapp listen host')
-gflags.DEFINE_integer('wsapi_port', 8080, 'webapp listen port')
 
+CONF = cfg.CONF
+CONF.register_cli_opts([
+    cfg.StrOpt('wsapi-host', default='', help='webapp listen host'),
+    cfg.IntOpt('wsapi-port', default=8080, help='webapp listen port')
+])
 
 HEX_PATTERN = r'0x[0-9a-z]+'
 DIGIT_PATTERN = r'[1-9][0-9]*'
@@ -52,7 +51,6 @@ class ControllerBase(object):
             if attr in kwargs:
                 del kwargs[attr]
 
-        # LOG.debug('kwargs %s', kwargs)
         return getattr(self, action)(req, **kwargs)
 
 
@@ -65,9 +63,6 @@ class WSGIApplication(object):
 
     @webob.dec.wsgify
     def __call__(self, req):
-        # LOG.debug('mapper %s', self.mapper)
-        # LOG.debug('req: %s\n', req)
-        # LOG.debug('\nreq.environ: %s', req.environ)
         match = self.mapper.match(environ=req.environ)
 
         if not match:
@@ -87,7 +82,7 @@ class WSGIApplication(object):
 
 class WSGIServer(pywsgi.WSGIServer):
     def __init__(self, application, **config):
-        super(WSGIServer, self).__init__((FLAGS.wsapi_host, FLAGS.wsapi_port),
+        super(WSGIServer, self).__init__((CONF.wsapi_host, CONF.wsapi_port),
                                          application, **config)
 
     def __call__(self):
