@@ -15,7 +15,7 @@
 # limitations under the License.
 
 import contextlib
-import gflags
+from oslo.config import cfg
 import logging
 import gevent
 import traceback
@@ -25,6 +25,8 @@ import ssl
 from gevent.server import StreamServer
 from gevent.queue import Queue
 
+import ryu.base.app_manager
+
 from ryu.ofproto import ofproto_common
 from ryu.ofproto import ofproto_parser
 from ryu.ofproto import ofproto_v1_0
@@ -35,21 +37,22 @@ from ryu.ofproto import ofproto_v1_3
 from ryu.ofproto import ofproto_v1_3_parser
 from ryu.ofproto import nx_match
 
-from ryu.controller import dispatcher
 from ryu.controller import handler
 from ryu.controller import ofp_event
 
 LOG = logging.getLogger('ryu.controller.controller')
 
-FLAGS = gflags.FLAGS
-gflags.DEFINE_string('ofp_listen_host', '', 'openflow listen host')
-gflags.DEFINE_integer('ofp_tcp_listen_port', ofproto_common.OFP_TCP_PORT,
-                      'openflow tcp listen port')
-gflags.DEFINE_integer('ofp_ssl_listen_port', ofproto_common.OFP_SSL_PORT,
-                      'openflow ssl listen port')
-gflags.DEFINE_string('ctl_privkey', None, 'controller private key')
-gflags.DEFINE_string('ctl_cert', None, 'controller certificate')
-gflags.DEFINE_string('ca_certs', None, 'CA certificates')
+CONF = cfg.CONF
+CONF.register_cli_opts([
+    cfg.StrOpt('ofp-listen-host', default='', help='openflow listen host'),
+    cfg.IntOpt('ofp-tcp-listen-port', default=ofproto_common.OFP_TCP_PORT,
+               help='openflow tcp listen port'),
+    cfg.IntOpt('ofp-ssl-listen-port', default=ofproto_common.OFP_SSL_PORT,
+               help='openflow ssl listen port'),
+    cfg.StrOpt('ctl-privkey', default=None, help='controller private key'),
+    cfg.StrOpt('ctl-cert', default=None, help='controller certificate'),
+    cfg.StrOpt('ca-certs', default=None, help='CA certificates')
+])
 
 
 class OpenFlowController(object):
@@ -62,26 +65,26 @@ class OpenFlowController(object):
         self.server_loop()
 
     def server_loop(self):
-        if FLAGS.ctl_privkey and FLAGS.ctl_cert is not None:
-            if FLAGS.ca_certs is not None:
-                server = StreamServer((FLAGS.ofp_listen_host,
-                                       FLAGS.ofp_ssl_listen_port),
+        if CONF.ctl_privkey is not None and CONF.ctl_cert is not None:
+            if CONF.ca_certs is not None:
+                server = StreamServer((CONF.ofp_listen_host,
+                                       CONF.ofp_ssl_listen_port),
                                       datapath_connection_factory,
-                                      keyfile=FLAGS.ctl_privkey,
-                                      certfile=FLAGS.ctl_cert,
+                                      keyfile=CONF.ctl_privkey,
+                                      certfile=CONF.ctl_cert,
                                       cert_reqs=ssl.CERT_REQUIRED,
-                                      ca_certs=FLAGS.ca_certs,
+                                      ca_certs=CONF.ca_certs,
                                       ssl_version=ssl.PROTOCOL_TLSv1)
             else:
-                server = StreamServer((FLAGS.ofp_listen_host,
-                                       FLAGS.ofp_ssl_listen_port),
+                server = StreamServer((CONF.ofp_listen_host,
+                                       CONF.ofp_ssl_listen_port),
                                       datapath_connection_factory,
-                                      keyfile=FLAGS.ctl_privkey,
-                                      certfile=FLAGS.ctl_cert,
+                                      keyfile=CONF.ctl_privkey,
+                                      certfile=CONF.ctl_cert,
                                       ssl_version=ssl.PROTOCOL_TLSv1)
         else:
-            server = StreamServer((FLAGS.ofp_listen_host,
-                                   FLAGS.ofp_tcp_listen_port),
+            server = StreamServer((CONF.ofp_listen_host,
+                                   CONF.ofp_tcp_listen_port),
                                   datapath_connection_factory)
 
         #LOG.debug('loop')
@@ -123,25 +126,22 @@ class Datapath(object):
         # prevent it from eating memory up
         self.send_q = Queue(16)
 
-        # circular reference self.ev_q.aux == self
-        self.ev_q = dispatcher.EventQueue(handler.QUEUE_NAME_OFP_MSG,
-                                          handler.HANDSHAKE_DISPATCHER,
-                                          self)
-
         self.set_version(max(self.supported_ofp_version))
         self.xid = random.randint(0, self.ofproto.MAX_XID)
         self.id = None  # datapath_id is unknown yet
         self.ports = None
         self.flow_format = ofproto_v1_0.NXFF_OPENFLOW10
+        self.ofp_brick = ryu.base.app_manager.lookup_service_brick('ofp_event')
+        self.set_state(handler.HANDSHAKE_DISPATCHER)
 
     def close(self):
-        """
-        Call this before discarding this datapath object
-        The circular refernce as self.ev_q.aux == self must be broken.
-        """
-        # tell this datapath is dead
-        self.ev_q.set_dispatcher(handler.DEAD_DISPATCHER)
-        self.ev_q.close()
+        self.set_state(handler.DEAD_DISPATCHER)
+
+    def set_state(self, state):
+        self.state = state
+        ev = ofp_event.EventOFPStateChange(self)
+        ev.state = state
+        self.ofp_brick.send_event_to_observers(ev, state)
 
     def set_version(self, version):
         assert version in self.supported_ofp_version
@@ -169,7 +169,14 @@ class Datapath(object):
                 msg = ofproto_parser.msg(self,
                                          version, msg_type, msg_len, xid, buf)
                 #LOG.debug('queue msg %s cls %s', msg, msg.__class__)
-                self.ev_q.queue(ofp_event.ofp_msg_to_ev(msg))
+                ev = ofp_event.ofp_msg_to_ev(msg)
+                self.ofp_brick.send_event_to_observers(ev, self.state)
+
+                handlers = [handler for handler in
+                            self.ofp_brick.get_handlers(ev) if self.state in
+                            handler.dispatchers]
+                for handler in handlers:
+                    handler(ev)
 
                 buf = buf[required_len:]
                 required_len = ofproto_common.OFP_HEADER_SIZE
@@ -185,12 +192,16 @@ class Datapath(object):
 
     @_deactivate
     def _send_loop(self):
-        while self.is_active:
-            buf = self.send_q.get()
-            self.socket.sendall(buf)
+        try:
+            while self.is_active:
+                buf = self.send_q.get()
+                self.socket.sendall(buf)
+        finally:
+            self.send_q = None
 
     def send(self, buf):
-        self.send_q.put(buf)
+        if self.send_q:
+            self.send_q.put(buf)
 
     def set_xid(self, msg):
         self.xid += 1
@@ -219,10 +230,6 @@ class Datapath(object):
             gevent.kill(send_thr)
             gevent.joinall([send_thr])
 
-    def send_ev(self, ev):
-        #LOG.debug('send_ev %s', ev)
-        self.ev_q.queue(ev)
-
     #
     # Utility methods for convenience
     #
@@ -290,6 +297,9 @@ class Datapath(object):
         self.send_msg(set_format)
         self.send_barrier()
 
+    def is_reserved_port(self, port_no):
+        return port_no > self.ofproto.OFPP_MAX
+
 
 def datapath_connection_factory(socket, address):
     LOG.debug('connected socket:%s address:%s', socket, address)
