@@ -35,6 +35,8 @@ def is_multicast(addr):
 def haddr_to_str(addr):
     """Format mac address in internal representation into human readable
     form"""
+    if addr is None:
+        return 'None'
     assert len(addr) == _HADDR_LEN
     return ':'.join('%02x' % ord(char) for char in addr)
 
@@ -44,7 +46,7 @@ def haddr_to_bin(string):
     internal representation"""
     hexes = string.split(':')
     if len(hexes) != _HADDR_LEN:
-        ValueError('Invalid format for mac address: %s' % string)
+        raise ValueError('Invalid format for mac address: %s' % string)
     return ''.join(chr(int(h, 16)) for h in hexes)
 
 
