@@ -101,7 +101,7 @@ def to_match(dp, attrs):
             wildcards &= ~ofp.OFPFW_IN_PORT
         elif key == 'dl_src':
             dl_src = haddr_to_bin(value)
-            wildcards &= ~ofp.OFPFW_DL_DST
+            wildcards &= ~ofp.OFPFW_DL_SRC
         elif key == 'dl_dst':
             dl_dst = haddr_to_bin(value)
             wildcards &= ~ofp.OFPFW_DL_DST
@@ -163,24 +163,46 @@ def match_to_str(m):
             'dl_vlan': m.dl_vlan,
             'dl_vlan_pcp': m.dl_vlan_pcp,
             'in_port': m.in_port,
-            'nw_dst': socket.inet_ntoa(struct.pack('!I', m.nw_dst)),
+            'nw_dst': nw_dst_to_str(m.wildcards, m.nw_dst),
             'nw_proto': m.nw_proto,
-            'nw_src': socket.inet_ntoa(struct.pack('!I', m.nw_src)),
+            'nw_src': nw_src_to_str(m.wildcards, m.nw_src),
             'tp_src': m.tp_src,
             'tp_dst': m.tp_dst}
 
 
+def nw_src_to_str(wildcards, addr):
+    ip = socket.inet_ntoa(struct.pack('!I', addr))
+    mask = 32 - ((wildcards & ofproto_v1_0.OFPFW_NW_SRC_MASK)
+                 >> ofproto_v1_0.OFPFW_NW_SRC_SHIFT)
+    if mask == 32:
+        mask = 0
+    if mask:
+        ip += '/%d' % mask
+    return ip
+
+
+def nw_dst_to_str(wildcards, addr):
+    ip = socket.inet_ntoa(struct.pack('!I', addr))
+    mask = 32 - ((wildcards & ofproto_v1_0.OFPFW_NW_DST_MASK)
+                 >> ofproto_v1_0.OFPFW_NW_DST_SHIFT)
+    if mask == 32:
+        mask = 0
+    if mask:
+        ip += '/%d' % mask
+    return ip
+
+
 def send_stats_request(dp, stats, waiters, msgs):
     dp.set_xid(stats)
-    waiters = waiters.setdefault(dp.id, {})
+    waiters_per_dp = waiters.setdefault(dp.id, {})
     lock = gevent.event.AsyncResult()
-    waiters[stats.xid] = (lock, msgs)
+    waiters_per_dp[stats.xid] = (lock, msgs)
     dp.send_msg(stats)
 
     try:
         lock.get(timeout=DEFAULT_TIMEOUT)
     except gevent.Timeout:
-        del waiters[dp.id][stats.xid]
+        del waiters_per_dp[stats.xid]
 
 
 def get_desc_stats(dp, waiters):
@@ -256,7 +278,7 @@ def get_port_stats(dp, waiters):
     return ports
 
 
-def push_flow_entry(dp, flow):
+def mod_flow_entry(dp, flow, cmd):
     cookie = int(flow.get('cookie', 0))
     priority = int(flow.get('priority',
                             dp.ofproto.OFP_DEFAULT_PRIORITY))
@@ -268,7 +290,7 @@ def push_flow_entry(dp, flow):
 
     flow_mod = dp.ofproto_parser.OFPFlowMod(
         datapath=dp, match=match, cookie=cookie,
-        command=dp.ofproto.OFPFC_ADD, idle_timeout=idle_timeout,
+        command=cmd, idle_timeout=idle_timeout,
         hard_timeout=hard_timeout, priority=priority, flags=flags,
         actions=actions)
 
