@@ -35,11 +35,11 @@ class RunTest(tester.TestFlowBase):
 
         self._verify = []
 
-    def add_apply_actions(self, dp, actions):
+    def add_apply_actions(self, dp, actions, match=None):
         inst = [dp.ofproto_parser.OFPInstructionActions(
                 dp.ofproto.OFPIT_APPLY_ACTIONS, actions)]
-
-        match = dp.ofproto_parser.OFPMatch()
+        if match is None:
+            match = dp.ofproto_parser.OFPMatch()
         m = dp.ofproto_parser.OFPFlowMod(dp, 0, 0, 0,
                                          dp.ofproto.OFPFC_ADD,
                                          0, 0, 0xff, 0xffffffff,
@@ -48,12 +48,12 @@ class RunTest(tester.TestFlowBase):
                                          0, match, inst)
         dp.send_msg(m)
 
-    def add_set_field_action(self, dp, field, value):
+    def add_set_field_action(self, dp, field, value, match=None):
         self._verify = [dp.ofproto.OFPAT_SET_FIELD,
                         'field', field, value]
         f = dp.ofproto_parser.OFPMatchField.make(field, value)
         actions = [dp.ofproto_parser.OFPActionSetField(f), ]
-        self.add_apply_actions(dp, actions)
+        self.add_apply_actions(dp, actions, match=match)
 
     def verify_default(self, dp, stats):
         verify = self._verify
@@ -176,109 +176,159 @@ class RunTest(tester.TestFlowBase):
         field = dp.ofproto.OXM_OF_VLAN_PCP
         value = 3
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_vlan_vid(1)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_nw_dscp(self, dp):
         field = dp.ofproto.OXM_OF_IP_DSCP
         value = 32
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_IP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_nw_ecn(self, dp):
         field = dp.ofproto.OXM_OF_IP_ECN
         value = 1
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_IP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_ip_proto(self, dp):
         field = dp.ofproto.OXM_OF_IP_PROTO
         value = inet.IPPROTO_TCP
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_IP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_ipv4_src(self, dp):
         field = dp.ofproto.OXM_OF_IPV4_SRC
         ipv4_src = '192.168.3.92'
         value = self.ipv4_to_int(ipv4_src)
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_IP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_ipv4_dst(self, dp):
         field = dp.ofproto.OXM_OF_IPV4_DST
         ipv4_dst = '192.168.74.122'
         value = self.ipv4_to_int(ipv4_dst)
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_IP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_tcp_src(self, dp):
         field = dp.ofproto.OXM_OF_TCP_SRC
         value = 105
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_IP)
+        match.set_ip_proto(inet.IPPROTO_TCP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_tcp_dst(self, dp):
         field = dp.ofproto.OXM_OF_TCP_DST
         value = 75
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_IP)
+        match.set_ip_proto(inet.IPPROTO_TCP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_udp_src(self, dp):
         field = dp.ofproto.OXM_OF_UDP_SRC
         value = 197
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_IP)
+        match.set_ip_proto(inet.IPPROTO_UDP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_udp_dst(self, dp):
         field = dp.ofproto.OXM_OF_UDP_DST
         value = 17
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_IP)
+        match.set_ip_proto(inet.IPPROTO_UDP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_icmpv4_type(self, dp):
         field = dp.ofproto.OXM_OF_ICMPV4_TYPE
         value = 8
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_ip_proto(inet.IPPROTO_ICMP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_icmpv4_code(self, dp):
         field = dp.ofproto.OXM_OF_ICMPV4_CODE
         value = 2
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_ip_proto(inet.IPPROTO_ICMP)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_arp_op(self, dp):
         field = dp.ofproto.OXM_OF_ARP_OP
         value = 2
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_ARP)
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_arp_spa(self, dp):
         field = dp.ofproto.OXM_OF_ARP_SPA
         nw_src = '192.168.132.179'
         value = self.ipv4_to_int(nw_src)
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_ARP)
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_arp_tpa(self, dp):
         field = dp.ofproto.OXM_OF_ARP_TPA
         nw_dst = '192.168.118.85'
         value = self.ipv4_to_int(nw_dst)
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_ARP)
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_arp_sha(self, dp):
         field = dp.ofproto.OXM_OF_ARP_SHA
         arp_sha = '50:29:e7:7f:6c:7f'
         value = self.haddr_to_bin(arp_sha)
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_ARP)
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_arp_tha(self, dp):
         field = dp.ofproto.OXM_OF_ARP_THA
         arp_tha = '71:c8:72:2f:47:fd'
         value = self.haddr_to_bin(arp_tha)
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_ARP)
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_ipv6_src(self, dp):
         field = dp.ofproto.OXM_OF_IPV6_SRC
@@ -335,15 +385,21 @@ class RunTest(tester.TestFlowBase):
 
     def test_action_set_field_mpls_label(self, dp):
         field = dp.ofproto.OXM_OF_MPLS_LABEL
-        value = 0x4cd41
+        value = 0x4c
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_MPLS)
+
+        self.add_set_field_action(dp, field, value, match)
 
     def test_action_set_field_mpls_tc(self, dp):
         field = dp.ofproto.OXM_OF_MPLS_TC
         value = 0b101
 
-        self.add_set_field_action(dp, field, value)
+        match = dp.ofproto_parser.OFPMatch()
+        match.set_dl_type(ether.ETH_TYPE_MPLS)
+
+        self.add_set_field_action(dp, field, value, match)
 
     # Test of Change-TTL Actions
     def test_action_set_mpls_ttl(self, dp):
@@ -358,6 +414,18 @@ class RunTest(tester.TestFlowBase):
         actions = [dp.ofproto_parser.OFPActionDecMplsTtl(), ]
         self.add_apply_actions(dp, actions)
 
+    def test_action_set_nw_ttl(self, dp):
+        nw_ttl = 64
+        self._verify = [dp.ofproto.OFPAT_SET_NW_TTL,
+                        'nw_ttl', nw_ttl]
+        actions = [dp.ofproto_parser.OFPActionSetNwTtl(nw_ttl), ]
+        self.add_apply_actions(dp, actions)
+
+    def test_action_dec_nw_ttl(self, dp):
+        self._verify = [dp.ofproto.OFPAT_DEC_NW_TTL]
+        actions = [dp.ofproto_parser.OFPActionDecNwTtl(), ]
+        self.add_apply_actions(dp, actions)
+
     def test_action_copy_ttl_out(self, dp):
         self._verify = [dp.ofproto.OFPAT_COPY_TTL_OUT]
         actions = [dp.ofproto_parser.OFPActionCopyTtlOut(), ]
@@ -367,3 +435,18 @@ class RunTest(tester.TestFlowBase):
         self._verify = [dp.ofproto.OFPAT_COPY_TTL_IN]
         actions = [dp.ofproto_parser.OFPActionCopyTtlIn(), ]
         self.add_apply_actions(dp, actions)
+
+    def is_supported(self, t):
+        unsupported = [
+            'test_action_set_field_ip_proto',
+            'test_action_set_field_dl_type',
+            'test_action_set_field_arp',
+            'test_action_set_field_ipv6',
+            'test_action_set_field_icmp',
+            'test_action_set_nw_ttl',
+        ]
+        for u in unsupported:
+            if t.find(u) != -1:
+                return False
+
+        return True
